package javax.swing.text.rtf;

import javax.swing.text.BadLocationException;
import javax.swing.text.StyleConstants;
import javax.swing.text.TabStop;
import javax.swing.text.AttributeSet;
import java.util.Vector;
import java.net.URL;
import java.io.StreamTokenizer;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.io.InputStream;
import java.io.IOException;
import java.util.Enumeration;
import javax.swing.text.SimpleAttributeSet;
import java.util.Hashtable;
import javax.swing.text.Style;
import java.awt.Color;
import javax.swing.text.MutableAttributeSet;
import java.util.Dictionary;
import javax.swing.text.StyledDocument;

class RTFReader extends RTFParser
{
    StyledDocument target;
    Dictionary<Object, Object> parserState;
    RTFReader.Destination rtfDestination;
    MutableAttributeSet documentAttributes;
    Dictionary<Integer, String> fontTable;
    Color[] colorTable;
    Style[] characterStyles;
    Style[] paragraphStyles;
    Style[] sectionStyles;
    int rtfversion;
    boolean ignoreGroupIfUnknownKeyword;
    int skippingCharacters;
    private static Dictionary<String, RTFAttribute> straightforwardAttributes;
    private MockAttributeSet mockery;
    static Dictionary<String, String> textKeywords;
    static final String TabAlignmentKey = "tab_alignment";
    static final String TabLeaderKey = "tab_leader";
    static Dictionary<String, char[]> characterSets;
    static boolean useNeXTForAnsi;
    
    public RTFReader(StyledDocument target) {
        this.target = target;
        this.parserState = new Hashtable();
        this.fontTable = new Hashtable();
        this.rtfversion = -1;
        this.mockery = new MockAttributeSet();
        this.documentAttributes = new SimpleAttributeSet();
    }
    
    @Override
    public void handleBinaryBlob(byte[] array) {
        if (this.skippingCharacters > 0) {
            --this.skippingCharacters;
            return;
        }
        return;
    }
    
    @Override
    public void handleText(String substring) {
        if (this.skippingCharacters > 0) {
            if (this.skippingCharacters >= substring.length()) {
                this.skippingCharacters -= substring.length();
                return;
            }
            substring = substring.substring(this.skippingCharacters);
            this.skippingCharacters = 0;
        }
        if (this.rtfDestination != null) {
            this.rtfDestination.handleText(substring);
            return;
        }
        this.warning("Text with no destination. oops.");
        return;
    }
    
    Color defaultColor() {
        return Color.black;
    }
    
    @Override
    public void begingroup() {
        if (this.skippingCharacters > 0) {
            this.skippingCharacters = 0;
        }
        Object value = this.parserState.get("_savedState");
        if (value != null) {
            this.parserState.remove("_savedState");
        }
        Dictionary<String, Object> dictionary = (Dictionary)((Hashtable)this.parserState).clone();
        if (value != null) {
            dictionary.put("_savedState", value);
        }
        this.parserState.put("_savedState", dictionary);
        if (this.rtfDestination != null) {
            this.rtfDestination.begingroup();
        }
    }
    
    @Override
    public void endgroup() {
        if (this.skippingCharacters > 0) {
            this.skippingCharacters = 0;
        }
        Dictionary<Object, Object> parserState = (Dictionary)this.parserState.get("_savedState");
        RTFReader.Destination rtfDestination = (RTFReader.Destination)parserState.get("dst");
        if (rtfDestination != this.rtfDestination) {
            this.rtfDestination.close();
            this.rtfDestination = rtfDestination;
        }
        Dictionary<Object, Object> parserState2 = this.parserState;
        this.parserState = parserState;
        if (this.rtfDestination != null) {
            this.rtfDestination.endgroup(parserState2);
        }
    }
    
    protected void setRTFDestination(RTFReader.Destination rtfDestination) {
        Dictionary<Object, Object> dictionary = (Dictionary)this.parserState.get("_savedState");
        if (dictionary != null && this.rtfDestination != dictionary.get("dst")) {
            this.warning("Warning, RTF destination overridden, invalid RTF.");
            this.rtfDestination.close();
        }
        this.rtfDestination = rtfDestination;
        this.parserState.put("dst", this.rtfDestination);
    }
    
    @Override
    public void close() throws IOException {
        Enumeration<Object> attributeNames = this.documentAttributes.getAttributeNames();
        while (attributeNames.hasMoreElements()) {
            Object nextElement = attributeNames.nextElement();
            this.target.putProperty(nextElement, this.documentAttributes.getAttribute(nextElement));
        }
        this.warning("RTF filter done.");
        super.close();
    }
    
    @Override
    public boolean handleKeyword(String s) {
        int ignoreGroupIfUnknownKeyword = this.ignoreGroupIfUnknownKeyword;
        if (this.skippingCharacters > 0) {
            --this.skippingCharacters;
            return true;
        }
        this.ignoreGroupIfUnknownKeyword = false;
        String s2;
        if ((s2 = (String)RTFReader.textKeywords.get(s)) != null) {
            this.handleText(s2);
            return true;
        }
        if (s.equals("fonttbl")) {
            this.setRTFDestination(new RTFReader.FonttblDestination());
            return true;
        }
        if (s.equals("colortbl")) {
            this.setRTFDestination(new RTFReader.ColortblDestination());
            return true;
        }
        if (s.equals("stylesheet")) {
            this.setRTFDestination(new RTFReader.StylesheetDestination());
            return true;
        }
        if (s.equals("info")) {
            this.setRTFDestination(new RTFReader.InfoDestination());
            return false;
        }
        if (s.equals("mac")) {
            this.setCharacterSet("mac");
            return true;
        }
        if (s.equals("ansi")) {
            if (RTFReader.useNeXTForAnsi) {
                this.setCharacterSet("NeXT");
            }
            else {
                this.setCharacterSet("ansi");
            }
            return true;
        }
        else {
            if (s.equals("next")) {
                this.setCharacterSet("NeXT");
                return true;
            }
            if (s.equals("pc")) {
                this.setCharacterSet("cpg437");
                return true;
            }
            if (s.equals("pca")) {
                this.setCharacterSet("cpg850");
                return true;
            }
            if (s.equals("*")) {
                return this.ignoreGroupIfUnknownKeyword = true;
            }
            if (this.rtfDestination != null && this.rtfDestination.handleKeyword(s)) {
                return true;
            }
            if (s.equals("aftncn") || s.equals("aftnsep") || s.equals("aftnsepc") || s.equals("annotation") || s.equals("atnauthor") || s.equals("atnicn") || s.equals("atnid") || s.equals("atnref") || s.equals("atntime") || s.equals("atrfend") || s.equals("atrfstart") || s.equals("bkmkend") || s.equals("bkmkstart") || s.equals("datafield") || s.equals("do") || s.equals("dptxbxtext") || s.equals("falt") || s.equals("field") || s.equals("file") || s.equals("filetbl") || s.equals("fname") || s.equals("fontemb") || s.equals("fontfile") || s.equals("footer") || s.equals("footerf") || s.equals("footerl") || s.equals("footerr") || s.equals("footnote") || s.equals("ftncn") || s.equals("ftnsep") || s.equals("ftnsepc") || s.equals("header") || s.equals("headerf") || s.equals("headerl") || s.equals("headerr") || s.equals("keycode") || s.equals("nextfile") || s.equals("object") || s.equals("pict") || s.equals("pn") || s.equals("pnseclvl") || s.equals("pntxtb") || s.equals("pntxta") || s.equals("revtbl") || s.equals("rxe") || s.equals("tc") || s.equals("template") || s.equals("txe") || s.equals("xe")) {
                ignoreGroupIfUnknownKeyword = true;
            }
            if (!(ignoreGroupIfUnknownKeyword == 0)) {
                this.setRTFDestination(new RTFReader.DiscardingDestination());
            }
            return false;
        }
    }
    
    @Override
    public boolean handleKeyword(String s, int n) {
        int ignoreGroupIfUnknownKeyword = this.ignoreGroupIfUnknownKeyword;
        if (this.skippingCharacters > 0) {
            --this.skippingCharacters;
            return true;
        }
        this.ignoreGroupIfUnknownKeyword = false;
        if (s.equals("uc")) {
            this.parserState.put("UnicodeSkip", Integer.valueOf(n));
            return true;
        }
        if (s.equals("u")) {
            if (n < 0) {
                n += 65536;
            }
            super.handleText((char)n);
            Number n2 = (Number)(Number)this.parserState.get("UnicodeSkip");
            if (n2 != null) {
                this.skippingCharacters = n2.intValue();
            }
            else {
                this.skippingCharacters = 1;
            }
            return true;
        }
        else {
            if (s.equals("rtf")) {
                this.rtfversion = n;
                this.setRTFDestination(new RTFReader.DocumentDestination());
                return true;
            }
            if (s.startsWith("NeXT") || s.equals("private")) {
                ignoreGroupIfUnknownKeyword = true;
            }
            if (this.rtfDestination != null && this.rtfDestination.handleKeyword(s, n)) {
                return true;
            }
            if (!(ignoreGroupIfUnknownKeyword == 0)) {
                this.setRTFDestination(new RTFReader.DiscardingDestination());
            }
            return false;
        }
    }
    
    private void setTargetAttribute(String s, Object o) {
    }
    
    public void setCharacterSet(String s) {
        Object characterSet;
        try {
            characterSet = getCharacterSet(s);
        }
        catch (Exception obj) {
            this.warning("Exception loading RTF character set \"" + s + "\": " + obj);
            characterSet = null;
        }
        if (characterSet != null) {
            this.translationTable = (char[])(char[])characterSet;
        }
        else {
            this.warning("Unknown RTF character set \"" + s + "\"");
            if (!s.equals("ansi")) {
                try {
                    this.translationTable = (char[])(char[])getCharacterSet("ansi");
                }
                catch (IOException ex) {
                    throw new InternalError("RTFReader: Unable to find character set resources (" + ex + ")", ex);
                }
            }
        }
        this.setTargetAttribute("rtfCharacterSet", s);
    }
    
    public static void defineCharacterSet(String s, char[] array) {
        if (array.length < 256) {
            throw new IllegalArgumentException("Translation table must have 256 entries.");
        }
        RTFReader.characterSets.put(s, array);
        return;
    }
    
    public static Object getCharacterSet(final String s) throws IOException {
        char[] charset = (char[])RTFReader.characterSets.get(s);
        if (charset == null) {
            charset = readCharset((InputStream)AccessController.doPrivileged(new PrivilegedAction<InputStream>() {
                @Override
                public InputStream run() {
                    return RTFReader.class.getResourceAsStream("charsets/" + s + ".txt");
                }
            }));
            defineCharacterSet(s, charset);
            return charset;
        }
        return charset;
    }
    
    static char[] readCharset(InputStream in) throws IOException {
        char[] array = new char[256];
        StreamTokenizer streamTokenizer = new StreamTokenizer(new BufferedReader(new InputStreamReader(in, "ISO-8859-1")));
        streamTokenizer.eolIsSignificant(false);
        streamTokenizer.commentChar(35);
        streamTokenizer.slashSlashComments(true);
        streamTokenizer.slashStarComments(true);
        int i = 0;
        while (i < 256) {
            int nextToken;
            try {
                nextToken = streamTokenizer.nextToken();
            }
            catch (Exception obj) {
                throw new IOException("Unable to read from character set file (" + obj + ")");
            }
            if (nextToken != -2) {
                throw new IOException("Unexpected token in character set file");
            }
            array[i] = (char)(int)streamTokenizer.nval;
            ++i;
        }
        return array;
    }
    
    static char[] readCharset(URL url) throws IOException {
        return readCharset(url.openStream());
    }
    
    static {
        RTFReader.straightforwardAttributes = RTFAttributes.attributesByKeyword();
        RTFReader.textKeywords = null;
        (RTFReader.textKeywords = new Hashtable()).put("\\", "\\");
        RTFReader.textKeywords.put("{", "{");
        RTFReader.textKeywords.put("}", "}");
        RTFReader.textKeywords.put(" ", " ");
        RTFReader.textKeywords.put("~", " ");
        RTFReader.textKeywords.put("_", "‑");
        RTFReader.textKeywords.put("bullet", "•");
        RTFReader.textKeywords.put("emdash", "—");
        RTFReader.textKeywords.put("emspace", "\u2003");
        RTFReader.textKeywords.put("endash", "–");
        RTFReader.textKeywords.put("enspace", "\u2002");
        RTFReader.textKeywords.put("ldblquote", "“");
        RTFReader.textKeywords.put("lquote", "‘");
        RTFReader.textKeywords.put("ltrmark", "\u200e");
        RTFReader.textKeywords.put("rdblquote", "”");
        RTFReader.textKeywords.put("rquote", "’");
        RTFReader.textKeywords.put("rtlmark", "\u200f");
        RTFReader.textKeywords.put("tab", "\t");
        RTFReader.textKeywords.put("zwj", "\u200d");
        RTFReader.textKeywords.put("zwnj", "\u200c");
        RTFReader.textKeywords.put("-", "‧");
        RTFReader.useNeXTForAnsi = false;
        RTFReader.characterSets = new Hashtable();
    }
    
    class DiscardingDestination implements RTFReader.Destination
    {
        @Override
        public void handleBinaryBlob(byte[] array) {
        }
        
        @Override
        public void handleText(String s) {
        }
        
        @Override
        public boolean handleKeyword(String s) {
            return true;
        }
        
        @Override
        public boolean handleKeyword(String s, int n) {
            return true;
        }
        
        @Override
        public void begingroup() {
        }
        
        @Override
        public void endgroup(Dictionary dictionary) {
        }
        
        @Override
        public void close() {
        }
    }
    
    class FonttblDestination implements RTFReader.Destination
    {
        int nextFontNumber;
        Integer fontNumberKey;
        String nextFontFamily;
        
        FonttblDestination() {
            this.fontNumberKey = null;
        }
        
        @Override
        public void handleBinaryBlob(byte[] array) {
        }
        
        @Override
        public void handleText(String s) {
            int index = s.indexOf(59);
            String str;
            if (index > -1) {
                str = s.substring(0, index);
            }
            else {
                str = s;
            }
            if (this.nextFontNumber == -1 && this.fontNumberKey != null) {
                str = (String)RTFReader.this.fontTable.get(this.fontNumberKey) + str;
            }
            else {
                this.fontNumberKey = Integer.valueOf(this.nextFontNumber);
            }
            RTFReader.this.fontTable.put(this.fontNumberKey, str);
            this.nextFontNumber = -1;
            this.nextFontFamily = null;
        }
        
        @Override
        public boolean handleKeyword(String s) {
            if (s.charAt(0) == 'f') {
                this.nextFontFamily = s.substring(1);
                return true;
            }
            return false;
        }
        
        @Override
        public boolean handleKeyword(String s, int nextFontNumber) {
            if (s.equals("f")) {
                this.nextFontNumber = nextFontNumber;
                return true;
            }
            return false;
        }
        
        @Override
        public void begingroup() {
        }
        
        @Override
        public void endgroup(Dictionary dictionary) {
        }
        
        @Override
        public void close() {
            Enumeration<Integer> keys = RTFReader.this.fontTable.keys();
            RTFReader.this.warning("Done reading font table.");
            while (keys.hasMoreElements()) {
                Integer obj = (Integer)keys.nextElement();
                RTFReader.this.warning("Number " + obj + ": " + (String)RTFReader.this.fontTable.get(obj));
            }
        }
    }
    
    class ColortblDestination implements RTFReader.Destination
    {
        int red;
        int green;
        int blue;
        Vector<Color> proTemTable;
        
        public ColortblDestination() {
            this.red = 0;
            this.green = 0;
            this.blue = 0;
            this.proTemTable = new Vector();
        }
        
        @Override
        public void handleText(String s) {
            for (int i = 0; i < s.length(); ++i) {
                if (s.charAt(i) == ';') {
                    this.proTemTable.addElement(new Color(this.red, this.green, this.blue));
                }
            }
        }
        
        @Override
        public void close() {
            int size = this.proTemTable.size();
            RTFReader.this.warning("Done reading color table, " + size + " entries.");
            RTFReader.this.colorTable = new Color[size];
            this.proTemTable.copyInto(RTFReader.this.colorTable);
        }
        
        @Override
        public boolean handleKeyword(String s, int red) {
            if (s.equals("red")) {
                this.red = red;
            }
            else if (s.equals("green")) {
                this.green = red;
            }
            else if (s.equals("blue")) {
                this.blue = red;
            }
            else {
                return false;
            }
            return true;
        }
        
        @Override
        public boolean handleKeyword(String s) {
            return false;
        }
        
        @Override
        public void begingroup() {
        }
        
        @Override
        public void endgroup(Dictionary dictionary) {
        }
        
        @Override
        public void handleBinaryBlob(byte[] array) {
        }
    }
    
    class StylesheetDestination extends RTFReader.DiscardingDestination implements RTFReader.Destination
    {
        Dictionary<Integer, RTFReader.StylesheetDestination.StyleDefiningDestination> definedStyles;
        
        public StylesheetDestination() {
            this.definedStyles = new Hashtable();
        }
        
        @Override
        public void begingroup() {
            RTFReader.this.setRTFDestination(new RTFReader.StylesheetDestination.StyleDefiningDestination());
        }
        
        @Override
        public void close() {
            Vector<Object> vector = new Vector();
            Vector<Object> vector2 = new Vector();
            Vector<Style> vector3 = new Vector();
            Enumeration<RTFReader.StylesheetDestination.StyleDefiningDestination> elements = this.definedStyles.elements();
            while (elements.hasMoreElements()) {
                RTFReader.StylesheetDestination.StyleDefiningDestination styleDefiningDestination = (RTFReader.StylesheetDestination.StyleDefiningDestination)elements.nextElement();
                Style realize = styleDefiningDestination.realize();
                RTFReader.this.warning("Style " + styleDefiningDestination.number + " (" + styleDefiningDestination.styleName + "): " + realize);
                String s = (String)realize.getAttribute("style:type");
                if (s.equals("section")) {
                }
                else if (s.equals("character")) {
                    vector3 = vector;
                }
                else {
                    vector3 = vector2;
                }
                if (vector3.size() <= styleDefiningDestination.number) {
                    vector3.setSize(styleDefiningDestination.number + 1);
                }
                vector3.setElementAt(realize, styleDefiningDestination.number);
            }
            if (!vector.isEmpty()) {
                Style[] array = new Style[vector.size()];
                vector.copyInto(array);
                RTFReader.this.characterStyles = array;
            }
            if (!vector2.isEmpty()) {
                Style[] array2 = new Style[vector2.size()];
                vector2.copyInto(array2);
                RTFReader.this.paragraphStyles = array2;
            }
            if (!vector3.isEmpty()) {
                Style[] array3 = new Style[vector3.size()];
                vector3.copyInto(array3);
                RTFReader.this.sectionStyles = array3;
            }
        }
        
        class StyleDefiningDestination extends RTFReader.AttributeTrackingDestination implements RTFReader.Destination
        {
            final int STYLENUMBER_NONE = 222;
            boolean additive;
            boolean characterStyle;
            boolean sectionStyle;
            public String styleName;
            public int number;
            int basedOn;
            int nextStyle;
            boolean hidden;
            Style realizedStyle;
            
            public StyleDefiningDestination() {
                RTFReader.StylesheetDestination.this.this$0.super();
                this.additive = false;
                this.characterStyle = false;
                this.sectionStyle = false;
                this.styleName = null;
                this.number = 0;
                this.basedOn = 222;
                this.nextStyle = 222;
                this.hidden = false;
            }
            
            @Override
            public void handleText(String s) {
                if (this.styleName != null) {
                    this.styleName += s;
                }
                else {
                    this.styleName = s;
                }
            }
            
            @Override
            public void close() {
                int endIndex = (this.styleName == null) ? 0 : this.styleName.indexOf(59);
                if (endIndex > 0) {
                    this.styleName = this.styleName.substring(0, endIndex);
                }
                RTFReader.StylesheetDestination.this.definedStyles.put(Integer.valueOf(this.number), this);
                super.close();
            }
            
            @Override
            public boolean handleKeyword(String s) {
                if (s.equals("additive")) {
                    return this.additive = true;
                }
                if (s.equals("shidden")) {
                    return this.hidden = true;
                }
                return super.handleKeyword(s);
            }
            
            @Override
            public boolean handleKeyword(String s, int number) {
                if (s.equals("s")) {
                    this.characterStyle = false;
                    this.sectionStyle = false;
                    this.number = number;
                }
                else if (s.equals("cs")) {
                    this.characterStyle = true;
                    this.sectionStyle = false;
                    this.number = number;
                }
                else if (s.equals("ds")) {
                    this.characterStyle = false;
                    this.sectionStyle = true;
                    this.number = number;
                }
                else if (s.equals("sbasedon")) {
                    this.basedOn = number;
                }
                else if (s.equals("snext")) {
                    this.nextStyle = number;
                }
                else {
                    return super.handleKeyword(s, number);
                }
                return true;
            }
            
            public Style realize() {
                Style realize = null;
                Style realize2 = null;
                if (this.realizedStyle != null) {
                    return this.realizedStyle;
                }
                if (this.basedOn != 222) {
                    RTFReader.StylesheetDestination.StyleDefiningDestination styleDefiningDestination = (RTFReader.StylesheetDestination.StyleDefiningDestination)RTFReader.StylesheetDestination.this.definedStyles.get(Integer.valueOf(this.basedOn));
                    if (styleDefiningDestination != null && styleDefiningDestination != this) {
                        realize = styleDefiningDestination.realize();
                    }
                }
                this.realizedStyle = RTFReader.this.target.addStyle(this.styleName, realize);
                if (this.characterStyle) {
                    this.realizedStyle.addAttributes(this.currentTextAttributes());
                    this.realizedStyle.addAttribute("style:type", "character");
                }
                else if (this.sectionStyle) {
                    this.realizedStyle.addAttributes(this.currentSectionAttributes());
                    this.realizedStyle.addAttribute("style:type", "section");
                }
                else {
                    this.realizedStyle.addAttributes(this.currentParagraphAttributes());
                    this.realizedStyle.addAttribute("style:type", "paragraph");
                }
                if (this.nextStyle != 222) {
                    RTFReader.StylesheetDestination.StyleDefiningDestination styleDefiningDestination2 = (RTFReader.StylesheetDestination.StyleDefiningDestination)RTFReader.StylesheetDestination.this.definedStyles.get(Integer.valueOf(this.nextStyle));
                    if (styleDefiningDestination2 != null) {
                        realize2 = styleDefiningDestination2.realize();
                    }
                }
                if (realize2 != null) {
                    this.realizedStyle.addAttribute("style:nextStyle", realize2);
                }
                this.realizedStyle.addAttribute("style:additive", Boolean.valueOf(this.additive));
                this.realizedStyle.addAttribute("style:hidden", Boolean.valueOf(this.hidden));
                return this.realizedStyle;
            }
        }
    }
    
    class InfoDestination extends RTFReader.DiscardingDestination implements RTFReader.Destination
    {
    }
    
    abstract class AttributeTrackingDestination implements RTFReader.Destination
    {
        MutableAttributeSet characterAttributes;
        MutableAttributeSet paragraphAttributes;
        MutableAttributeSet sectionAttributes;
        
        public AttributeTrackingDestination() {
            this.characterAttributes = this.rootCharacterAttributes();
            RTFReader.this.parserState.put("chr", this.characterAttributes);
            this.paragraphAttributes = this.rootParagraphAttributes();
            RTFReader.this.parserState.put("pgf", this.paragraphAttributes);
            this.sectionAttributes = this.rootSectionAttributes();
            RTFReader.this.parserState.put("sec", this.sectionAttributes);
        }
        
        @Override
        public abstract void handleText(String p0);
        
        @Override
        public void handleBinaryBlob(byte[] array) {
            RTFReader.this.warning("Unexpected binary data in RTF file.");
        }
        
        @Override
        public void begingroup() {
            MutableAttributeSet currentTextAttributes = this.currentTextAttributes();
            MutableAttributeSet currentParagraphAttributes = this.currentParagraphAttributes();
            AttributeSet currentSectionAttributes = this.currentSectionAttributes();
            (this.characterAttributes = new SimpleAttributeSet()).addAttributes(currentTextAttributes);
            RTFReader.this.parserState.put("chr", this.characterAttributes);
            (this.paragraphAttributes = new SimpleAttributeSet()).addAttributes(currentParagraphAttributes);
            RTFReader.this.parserState.put("pgf", this.paragraphAttributes);
            (this.sectionAttributes = new SimpleAttributeSet()).addAttributes(currentSectionAttributes);
            RTFReader.this.parserState.put("sec", this.sectionAttributes);
        }
        
        @Override
        public void endgroup(Dictionary dictionary) {
            this.characterAttributes = (MutableAttributeSet)RTFReader.this.parserState.get("chr");
            this.paragraphAttributes = (MutableAttributeSet)RTFReader.this.parserState.get("pgf");
            this.sectionAttributes = (MutableAttributeSet)RTFReader.this.parserState.get("sec");
        }
        
        @Override
        public void close() {
        }
        
        @Override
        public boolean handleKeyword(String s) {
            if (s.equals("ulnone")) {
                return this.handleKeyword("ul", 0);
            }
            RTFAttribute rtfAttribute = (RTFAttribute)RTFReader.straightforwardAttributes.get(s);
            if (rtfAttribute != null) {
                int n;
                switch (rtfAttribute.domain()) {
                    case 0: {
                        n = rtfAttribute.set(this.characterAttributes);
                        break;
                    }
                    case 1: {
                        n = rtfAttribute.set(this.paragraphAttributes);
                        break;
                    }
                    case 2: {
                        n = rtfAttribute.set(this.sectionAttributes);
                        break;
                    }
                    case 4: {
                        RTFReader.this.mockery.backing = RTFReader.this.parserState;
                        n = rtfAttribute.set(RTFReader.this.mockery);
                        RTFReader.this.mockery.backing = null;
                        break;
                    }
                    case 3: {
                        n = rtfAttribute.set(RTFReader.this.documentAttributes);
                        break;
                    }
                    default: {
                        n = false;
                        break;
                    }
                }
                if (!(n == 0)) {
                    return true;
                }
            }
            if (s.equals("plain")) {
                this.resetCharacterAttributes();
                return true;
            }
            if (s.equals("pard")) {
                this.resetParagraphAttributes();
                return true;
            }
            if (s.equals("sectd")) {
                this.resetSectionAttributes();
                return true;
            }
            return false;
        }
        
        @Override
        public boolean handleKeyword(String s, int n) {
            int n2 = n != 0;
            if (s.equals("fc")) {
                s = "cf";
            }
            if (s.equals("f")) {
                RTFReader.this.parserState.put(s, Integer.valueOf(n));
                return true;
            }
            if (s.equals("cf")) {
                RTFReader.this.parserState.put(s, Integer.valueOf(n));
                return true;
            }
            RTFAttribute rtfAttribute = (RTFAttribute)RTFReader.straightforwardAttributes.get(s);
            if (rtfAttribute != null) {
                int n3;
                switch (rtfAttribute.domain()) {
                    case 0: {
                        n3 = rtfAttribute.set(this.characterAttributes, n);
                        break;
                    }
                    case 1: {
                        n3 = rtfAttribute.set(this.paragraphAttributes, n);
                        break;
                    }
                    case 2: {
                        n3 = rtfAttribute.set(this.sectionAttributes, n);
                        break;
                    }
                    case 4: {
                        RTFReader.this.mockery.backing = RTFReader.this.parserState;
                        n3 = rtfAttribute.set(RTFReader.this.mockery, n);
                        RTFReader.this.mockery.backing = null;
                        break;
                    }
                    case 3: {
                        n3 = rtfAttribute.set(RTFReader.this.documentAttributes, n);
                        break;
                    }
                    default: {
                        n3 = false;
                        break;
                    }
                }
                if (!(n3 == 0)) {
                    return true;
                }
            }
            if (s.equals("fs")) {
                StyleConstants.setFontSize(this.characterAttributes, n / 2);
                return true;
            }
            if (s.equals("sl")) {
                if (n == 1000) {
                    this.characterAttributes.removeAttribute(StyleConstants.LineSpacing);
                }
                else {
                    StyleConstants.setLineSpacing(this.characterAttributes, (float)n / 20.0f);
                }
                return true;
            }
            else if (s.equals("tx") || s.equals("tb")) {
                float pos = (float)n / 20.0f;
                int intValue = 0;
                Number n4 = (Number)(Number)RTFReader.this.parserState.get("tab_alignment");
                if (n4 != null) {
                    intValue = n4.intValue();
                }
                int intValue2 = 0;
                Number n5 = (Number)(Number)RTFReader.this.parserState.get("tab_leader");
                if (n5 != null) {
                    intValue2 = n5.intValue();
                }
                if (s.equals("tb")) {
                    intValue = 5;
                }
                RTFReader.this.parserState.remove("tab_alignment");
                RTFReader.this.parserState.remove("tab_leader");
                TabStop o = new TabStop(pos, intValue, intValue2);
                Dictionary<Object, Integer> dictionary = (Dictionary)RTFReader.this.parserState.get("_tabs");
                Integer n6;
                if (dictionary == null) {
                    dictionary = new Hashtable();
                    RTFReader.this.parserState.put("_tabs", dictionary);
                    n6 = Integer.valueOf(1);
                }
                else {
                    n6 = Integer.valueOf(1 + (int)dictionary.get("stop count"));
                }
                dictionary.put(n6, o);
                dictionary.put("stop count", n6);
                RTFReader.this.parserState.remove("_tabs_immutable");
                return true;
            }
            else {
                if (s.equals("s") && RTFReader.this.paragraphStyles != null) {
                    RTFReader.this.parserState.put("paragraphStyle", RTFReader.this.paragraphStyles[n]);
                    return true;
                }
                if (s.equals("cs") && RTFReader.this.characterStyles != null) {
                    RTFReader.this.parserState.put("characterStyle", RTFReader.this.characterStyles[n]);
                    return true;
                }
                if (s.equals("ds") && RTFReader.this.sectionStyles != null) {
                    RTFReader.this.parserState.put("sectionStyle", RTFReader.this.sectionStyles[n]);
                    return true;
                }
                return false;
            }
        }
        
        protected MutableAttributeSet rootCharacterAttributes() {
            SimpleAttributeSet set = new SimpleAttributeSet();
            StyleConstants.setItalic(set, false);
            StyleConstants.setBold(set, false);
            StyleConstants.setUnderline(set, false);
            StyleConstants.setForeground(set, RTFReader.this.defaultColor());
            return set;
        }
        
        protected MutableAttributeSet rootParagraphAttributes() {
            SimpleAttributeSet a = new SimpleAttributeSet();
            StyleConstants.setLeftIndent(a, 0.0f);
            StyleConstants.setRightIndent(a, 0.0f);
            StyleConstants.setFirstLineIndent(a, 0.0f);
            a.setResolveParent(RTFReader.this.target.getStyle("default"));
            return a;
        }
        
        protected MutableAttributeSet rootSectionAttributes() {
            return new SimpleAttributeSet();
        }
        
        MutableAttributeSet currentTextAttributes() {
            SimpleAttributeSet set = new SimpleAttributeSet(this.characterAttributes);
            Integer n = (Integer)RTFReader.this.parserState.get("f");
            String fam;
            if (n != null) {
                fam = (String)RTFReader.this.fontTable.get(n);
            }
            else {
                fam = null;
            }
            if (fam != null) {
                StyleConstants.setFontFamily(set, fam);
            }
            else {
                set.removeAttribute(StyleConstants.FontFamily);
            }
            if (RTFReader.this.colorTable != null) {
                Integer n2 = (Integer)RTFReader.this.parserState.get("cf");
                if (n2 != null) {
                    StyleConstants.setForeground(set, RTFReader.this.colorTable[(int)n2]);
                }
                else {
                    set.removeAttribute(StyleConstants.Foreground);
                }
            }
            if (RTFReader.this.colorTable != null) {
                Integer n3 = (Integer)RTFReader.this.parserState.get("cb");
                if (n3 != null) {
                    set.addAttribute(StyleConstants.Background, RTFReader.this.colorTable[(int)n3]);
                }
                else {
                    set.removeAttribute(StyleConstants.Background);
                }
            }
            Style resolveParent = (Style)RTFReader.this.parserState.get("characterStyle");
            if (resolveParent != null) {
                set.setResolveParent(resolveParent);
                return set;
            }
            return set;
        }
        
        MutableAttributeSet currentParagraphAttributes() {
            SimpleAttributeSet set = new SimpleAttributeSet(this.paragraphAttributes);
            TabStop[] array = (TabStop[])(TabStop[])RTFReader.this.parserState.get("_tabs_immutable");
            if (array == null) {
                Dictionary<Object, Object> dictionary = (Dictionary)RTFReader.this.parserState.get("_tabs");
                if (dictionary != null) {
                    int intValue = (int)dictionary.get("stop count");
                    array = new TabStop[intValue];
                    for (int i = 1; i <= intValue; ++i) {
                        array[i - 1] = (TabStop)dictionary.get(Integer.valueOf(i));
                    }
                    RTFReader.this.parserState.put("_tabs_immutable", array);
                }
            }
            if (array != null) {
                set.addAttribute("tabs", array);
            }
            Style resolveParent = (Style)RTFReader.this.parserState.get("paragraphStyle");
            if (resolveParent != null) {
                set.setResolveParent(resolveParent);
                return set;
            }
            return set;
        }
        
        public AttributeSet currentSectionAttributes() {
            SimpleAttributeSet set = new SimpleAttributeSet(this.sectionAttributes);
            Style resolveParent = (Style)RTFReader.this.parserState.get("sectionStyle");
            if (resolveParent != null) {
                set.setResolveParent(resolveParent);
                return set;
            }
            return set;
        }
        
        protected void resetCharacterAttributes() {
            this.handleKeyword("f", 0);
            this.handleKeyword("cf", 0);
            this.handleKeyword("fs", 24);
            Enumeration<Object> elements = RTFReader.straightforwardAttributes.elements();
            while (elements.hasMoreElements()) {
                RTFAttribute rtfAttribute = (RTFAttribute)elements.nextElement();
                if (rtfAttribute.domain() == 0) {
                    rtfAttribute.setDefault(this.characterAttributes);
                }
                else {
                    continue;
                }
            }
            this.handleKeyword("sl", 1000);
            RTFReader.this.parserState.remove("characterStyle");
        }
        
        protected void resetParagraphAttributes() {
            RTFReader.this.parserState.remove("_tabs");
            RTFReader.this.parserState.remove("_tabs_immutable");
            RTFReader.this.parserState.remove("paragraphStyle");
            StyleConstants.setAlignment(this.paragraphAttributes, 0);
            Enumeration<Object> elements = RTFReader.straightforwardAttributes.elements();
            while (elements.hasMoreElements()) {
                RTFAttribute rtfAttribute = (RTFAttribute)elements.nextElement();
                if (rtfAttribute.domain() == 1) {
                    rtfAttribute.setDefault(this.characterAttributes);
                }
                else {
                    continue;
                }
            }
        }
        
        protected void resetSectionAttributes() {
            Enumeration<Object> elements = RTFReader.straightforwardAttributes.elements();
            while (elements.hasMoreElements()) {
                RTFAttribute rtfAttribute = (RTFAttribute)elements.nextElement();
                if (rtfAttribute.domain() == 2) {
                    rtfAttribute.setDefault(this.characterAttributes);
                }
                else {
                    continue;
                }
            }
            RTFReader.this.parserState.remove("sectionStyle");
        }
    }
    
    abstract class TextHandlingDestination extends RTFReader.AttributeTrackingDestination implements RTFReader.Destination
    {
        boolean inParagraph;
        
        public TextHandlingDestination() {
            this.inParagraph = false;
        }
        
        @Override
        public void handleText(String s) {
            if (!this.inParagraph) {
                this.beginParagraph();
            }
            this.deliverText(s, this.currentTextAttributes());
        }
        
        abstract void deliverText(String p0, AttributeSet p1);
        
        @Override
        public void close() {
            if (this.inParagraph) {
                this.endParagraph();
            }
            super.close();
        }
        
        @Override
        public boolean handleKeyword(String s) {
            if (s.equals("\r") || s.equals("\n")) {
                s = "par";
            }
            if (s.equals("par")) {
                this.endParagraph();
                return true;
            }
            if (s.equals("sect")) {
                this.endSection();
                return true;
            }
            return super.handleKeyword(s);
        }
        
        protected void beginParagraph() {
            this.inParagraph = true;
        }
        
        protected void endParagraph() {
            this.finishParagraph(this.currentParagraphAttributes(), this.currentTextAttributes());
            this.inParagraph = false;
        }
        
        abstract void finishParagraph(AttributeSet p0, AttributeSet p1);
        
        abstract void endSection();
    }
    
    class DocumentDestination extends RTFReader.TextHandlingDestination implements RTFReader.Destination
    {
        public void deliverText(String s, AttributeSet set) {
            try {
                RTFReader.this.target.insertString(RTFReader.this.target.getLength(), s, this.currentTextAttributes());
            }
            catch (BadLocationException cause) {
                throw new InternalError(cause.getMessage(), cause);
            }
        }
        
        public void finishParagraph(AttributeSet set, AttributeSet set2) {
            int length = RTFReader.this.target.getLength();
            try {
                RTFReader.this.target.insertString(length, "\n", set2);
                RTFReader.this.target.setParagraphAttributes(length, 1, set, true);
            }
            catch (BadLocationException cause) {
                throw new InternalError(cause.getMessage(), cause);
            }
        }
        
        public void endSection() {
        }
    }
    
    interface Destination
    {
        public abstract void handleBinaryBlob(byte[] p0);
        
        public abstract void handleText(String p0);
        
        public abstract boolean handleKeyword(String p0);
        
        public abstract boolean handleKeyword(String p0, int p1);
        
        public abstract void begingroup();
        
        public abstract void endgroup(Dictionary p0);
        
        public abstract void close();
    }
}
